//! Rhai scripting integration for the SAL library
//!
//! This module provides integration with the Rhai scripting language,
//! allowing SAL functions to be called from Rhai scripts.

mod error;
mod os;
mod process;
mod buildah;

#[cfg(test)]
mod tests;

// Re-export common Rhai types for convenience
pub use rhai::{Array, Dynamic, Map, EvalAltResult, Engine};

// Re-export error module
pub use error::*;

// Re-export specific functions from modules to avoid name conflicts
pub use os::{
    register_os_module,
    // File system functions
    exist, find_file, find_files, find_dir, find_dirs, 
    delete, mkdir, file_size, rsync,
    // Download functions
    download, download_install
};

pub use process::{
    register_process_module,
    // Run functions
    run_command, run_silent, run_with_options, new_run_options,
    // Process management functions
    which, kill, process_list, process_get
};

pub use buildah::{
    register_buildah_module,
    // Container functions
    from, run, run_with_isolation, add, commit, remove, list,
    build_with_options, new_build_options,
    // Image functions
    images, image_remove, image_push, image_tag, image_pull,
    image_commit_with_options, new_commit_options,
    config_with_options, new_config_options
};

// Rename copy functions to avoid conflicts
pub use os::copy as os_copy;
pub use buildah::copy as buildah_copy;

/// Register all SAL modules with the Rhai engine
///
/// # Arguments
///
/// * `engine` - The Rhai engine to register the modules with
///
/// # Example
///
/// ```
/// use rhai::Engine;
/// use sal::rhai;
///
/// let mut engine = Engine::new();
/// rhai::register(&mut engine);
///
/// // Now you can use SAL functions in Rhai scripts
/// let result = engine.eval::<bool>("exist('some_file.txt')").unwrap();
/// ```
pub fn register(engine: &mut Engine) -> Result<(), Box<rhai::EvalAltResult>> {
    // Register OS module functions
    os::register_os_module(engine)?;
    
    // Register Process module functions
    process::register_process_module(engine)?;
    
    // Register Buildah module functions
    buildah::register_buildah_module(engine)?;
    
    // Future modules can be registered here
    
    Ok(())
}